@@ -24,7 +24,7 @@ from girder_jobs.models.job import Job
 
 from girder import logger
 from girder.constants import SortDir
-from girder.exceptions import FilePathException, ValidationException
+from girder.exceptions import FilePathException, GirderException, ValidationException
 from girder.models.assetstore import Assetstore
 from girder.models.file import File
 from girder.models.item import Item
@@ -318,6 +318,26 @@ class ImageItem(Item):
 
     def _getAndCacheImageOrData(
             self, item, imageFunc, checkAndCreate, keydict, pickleCache=False, **kwargs):
+        """
+        Get a file associated with an image that can be generated by a
+        function.
+
+        :param item: the idem to process.
+        :param imageFunc: the function to call to generate a file.
+        :param checkAndCreate: False to return the data, creating and caching
+            it if needed.  True to return True if the data is already in cache,
+            or to create the data, cache, and return it if not.  'nosave' to
+            return data from the cache if present, or generate the data but do
+            not return it if not in the cache.  'check' to just return True or
+            False to report if it is in the cache.
+        :param keydict: a dictionary of values to use for the cache key.
+        :param pickleCache: if True, the results of the function are pickled to
+            preserver them.  If Fales, the results can be saved as a file
+            directly.
+        :params **kwargs: passed to the tile source and to the imageFunc.  May
+            contain contentDisposition to determine how results are returned.
+        :returns:
+        """
         if 'fill' in keydict and (keydict['fill']).lower() == 'none':
             del keydict['fill']
         keydict = {k: v for k, v in keydict.items() if v is not None and not k.startswith('_')}
@@ -339,6 +359,8 @@ class ImageItem(Item):
                 data = File().open(existing).read()
                 return pickle.loads(data), 'application/octet-stream'
             return File().download(existing, contentDisposition=contentDisposition)
+        if checkAndCreate == 'check':
+            return False
         tileSource = self._loadTileSource(item, **kwargs)
         result = getattr(tileSource, imageFunc)(**kwargs)
         if result is None:
@@ -360,21 +382,24 @@ class ImageItem(Item):
                 pickleCache or isinstance(imageData, bytes))):
             dataStored = imageData if not pickleCache else pickle.dumps(imageData, protocol=4)
             # Save the data as a file
-            datafile = Upload().uploadFromFile(
-                io.BytesIO(dataStored), size=len(dataStored),
-                name='_largeImageThumbnail', parentType='item', parent=item,
-                user=None, mimeType=imageMime, attachParent=True)
-            if not len(dataStored) and 'received' in datafile:
-                datafile = Upload().finalizeUpload(
-                    datafile, Assetstore().load(datafile['assetstoreId']))
-            datafile.update({
-                'isLargeImageThumbnail' if not pickleCache else 'isLargeImageData': True,
-                'thumbnailKey': key,
-            })
-            # Ideally, we would check that the file is still wanted before we
-            # save it.  This is probably impossible without true transactions in
-            # Mongo.
-            File().save(datafile)
+            try:
+                datafile = Upload().uploadFromFile(
+                    io.BytesIO(dataStored), size=len(dataStored),
+                    name='_largeImageThumbnail', parentType='item', parent=item,
+                    user=None, mimeType=imageMime, attachParent=True)
+                if not len(dataStored) and 'received' in datafile:
+                    datafile = Upload().finalizeUpload(
+                        datafile, Assetstore().load(datafile['assetstoreId']))
+                datafile.update({
+                    'isLargeImageThumbnail' if not pickleCache else 'isLargeImageData': True,
+                    'thumbnailKey': key,
+                })
+                # Ideally, we would check that the file is still wanted before
+                # we save it.  This is probably impossible without true
+                # transactions in Mongo.
+                File().save(datafile)
+            except (GirderException, PermissionError):
+                logger.warning('Could not cache data for large image')
         return imageData, imageMime
 
     def removeThumbnailFiles(self, item, keep=0, sort=None, imageKey=None, **kwargs):
@@ -536,3 +561,28 @@ class ImageItem(Item):
         keydict = dict(kwargs, imageKey=imageKey)
         return self._getAndCacheImageOrData(
             item, 'getAssociatedImage', checkAndCreate, keydict, imageKey=imageKey, **kwargs)
+
+    def _scheduleTileFrames(self, item, tileFramesList, user):
+        """
+        Schedule generating tile frames in a local job.
+
+        :param item: the item.
+        :param tileFramesList: a list of dictionary of parameters to pass to
+            the tileFrames method.
+        :param user: the user owning the job.
+        """
+        job = Job().createLocalJob(
+            module='large_image_tasks.tasks',
+            function='cache_tile_frames_job',
+            kwargs={
+                'itemId': str(item['_id']),
+                'tileFramesList': tileFramesList,
+            },
+            title='Cache tileFrames',
+            type='large_image_cache_tile_frames',
+            user=user,
+            public=True,
+            asynchronous=True,
+        )
+        Job().scheduleJob(job)
+        return job
